@@ -27,7 +27,7 @@ struct IntrinsicProceduresAsASRNodes {
             intrinsics_present_in_ASR = {"size", "lbound", "ubound",
                 "transpose", "matmul", "pack", "transfer", "cmplx",
                 "dcmplx", "reshape", "ichar", "iachar", "char", "maxloc",
-                "null", "associated", "_lfortran_sqrt", "all", "len"};
+                "null", "associated", "all", "len"};
 
             kind_based_intrinsics = {"scan", "verify"};
         }
@@ -124,13 +124,10 @@ struct IntrinsicProcedures {
             {"erfc", {m_math, &eval_erfc, true}},
             {"abs", {m_math, &not_implemented, true}},  // Implemented using
             {"iabs", {m_math, &not_implemented, true}}, // IntrinsicFunction
-            {"sqrt", {m_math, &eval_sqrt, true}},
-            {"dsqrt", {m_math, &eval_dsqrt, true}},
             {"datan", {m_math, &eval_datan, true}},
             {"dabs", {m_math2, &eval_dabs, true}},
             {"dcos", {m_math, &eval_dcos, true}},
             {"dsin", {m_math, &eval_dsin, true}},
-            {"gamma", {m_math, &eval_gamma, true}},
             {"log_gamma", {m_math, &eval_log_gamma, true}},
             {"log10", {m_math, &eval_log10, true}},
             {"dlog10", {m_math, &eval_dlog10, true}},
@@ -539,7 +536,6 @@ TRIG(atanh)
 
 TRIG(exp)
 TRIG(log)
-TRIG(sqrt)
 
 TRIG2(exp, dexp)
 TRIG2(exp, sexp)
@@ -556,7 +552,6 @@ TRIG2(log, zlog)
 TRIG2(sin, dsin)
 TRIG2(cos, dcos)
 TRIG2(atan, datan)
-TRIG2(sqrt, dsqrt)
 
 
     static ASR::expr_t *eval_erf(Allocator &al, const Location &loc, Vec<ASR::expr_t*> &args) {
@@ -565,9 +560,6 @@ TRIG2(sqrt, dsqrt)
     static ASR::expr_t *eval_erfc(Allocator &al, const Location &loc, Vec<ASR::expr_t*> &args) {
         return eval_trig(al, loc, args, &erfc, nullptr);
     }
-    static ASR::expr_t *eval_gamma(Allocator &al, const Location &loc, Vec<ASR::expr_t*> &args) {
-        return eval_trig(al, loc, args, &tgamma, nullptr);
-    }
     static ASR::expr_t *eval_log_gamma(Allocator &al, const Location &loc, Vec<ASR::expr_t*> &args) {
         return eval_trig(al, loc, args, &lgamma, nullptr);
     }
