@@ -134,7 +134,8 @@ void extract_module_python(const ASR::TranslationUnit_t &m,
     }
 }
 
-void update_call_args(Allocator &al, SymbolTable *current_scope, bool implicit_interface) {
+void update_call_args(Allocator &al, SymbolTable *current_scope, bool implicit_interface,
+        std::map<std::string, ASR::symbol_t*> changed_external_function_symbol) {
     /*
         Iterate over body of program, check if there are any subroutine calls if yes, iterate over its args
         and update the args if they are equal to the old symbol
@@ -147,11 +148,34 @@ void update_call_args(Allocator &al, SymbolTable *current_scope, bool implicit_i
         This function updates `sub2` to use the new symbol `c` that is now a function, not a variable.
         Along with this, it also updates the args of `sub2` to use the new symbol `c` instead of the old one.
     */
-    class UpdateArgsVisitor : public PassUtils::PassVisitor<UpdateArgsVisitor>
+
+    class ArgsReplacer : public ASR::BaseExprReplacer<ArgsReplacer> {
+    public:
+        Allocator &al;
+        ASR::symbol_t* new_sym;
+
+        ArgsReplacer(Allocator &al_) : al(al_) {}
+
+        void replace_Var(ASR::Var_t* x) {
+            *current_expr = ASRUtils::EXPR(ASR::make_Var_t(al, x->base.base.loc, new_sym));
+        }
+    };
+
+    class ArgsVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArgsVisitor>
     {
         public:
+        Allocator &al;
         SymbolTable* scope = current_scope;
-        UpdateArgsVisitor(Allocator &al) : PassVisitor(al, nullptr) {}
+        ArgsReplacer replacer;
+        std::map<std::string, ASR::symbol_t*> &changed_external_function_symbol;
+        ArgsVisitor(Allocator &al_, std::map<std::string, ASR::symbol_t*> &changed_external_function_symbol_) : al(al_), replacer(al_),
+                    changed_external_function_symbol(changed_external_function_symbol_) {}
+
+        void call_replacer_(ASR::symbol_t* new_sym) {
+            replacer.current_expr = current_expr;
+            replacer.new_sym = new_sym;
+            replacer.replace_expr(*current_expr);
+        }
 
         ASR::symbol_t* fetch_sym(ASR::symbol_t* arg_sym_underlying) {
             ASR::symbol_t* sym = nullptr;
@@ -167,25 +191,58 @@ void update_call_args(Allocator &al, SymbolTable *current_scope, bool implicit_i
             return sym;
         }
 
+        void handle_Var(ASR::expr_t* arg_expr, ASR::expr_t** expr_to_replace) {
+            if (ASR::is_a<ASR::Var_t>(*arg_expr)) {
+                ASR::Var_t* arg_var = ASR::down_cast<ASR::Var_t>(arg_expr);
+                ASR::symbol_t* arg_sym = arg_var->m_v;
+                ASR::symbol_t* arg_sym_underlying = ASRUtils::symbol_get_past_external(arg_sym);
+                ASR::symbol_t* sym = fetch_sym(arg_sym_underlying);
+                if (sym != arg_sym) {
+                    ASR::expr_t** current_expr_copy = current_expr;
+                    current_expr = const_cast<ASR::expr_t**>((expr_to_replace));
+                    this->call_replacer_(sym);
+                    current_expr = current_expr_copy;
+                }
+            }
+        }
+
+
         void visit_SubroutineCall(const ASR::SubroutineCall_t& x) {
             ASR::SubroutineCall_t* subrout_call = (ASR::SubroutineCall_t*)(&x);
             for (size_t j = 0; j < subrout_call->n_args; j++) {
                 ASR::call_arg_t arg = subrout_call->m_args[j];
                 ASR::expr_t* arg_expr = arg.m_value;
-                if (ASR::is_a<ASR::Var_t>(*arg_expr)) {
-                    ASR::Var_t* arg_var = ASR::down_cast<ASR::Var_t>(arg_expr);
-                    ASR::symbol_t* arg_sym = arg_var->m_v;
-                    ASR::symbol_t* arg_sym_underlying = ASRUtils::symbol_get_past_external(arg_sym);
-                    ASR::symbol_t* sym = fetch_sym(arg_sym_underlying);
-                    if (sym != arg_sym) {
-                        subrout_call->m_args[j].m_value = ASRUtils::EXPR(ASR::make_Var_t(al, arg_expr->base.loc, sym));
-                    }
-                }
+                handle_Var(arg_expr, &(subrout_call->m_args[j].m_value));
+            }
+        }
+
+        void visit_FunctionCall(const ASR::FunctionCall_t& x) {
+            ASR::FunctionCall_t* func_call = (ASR::FunctionCall_t*)(&x);
+            for (size_t j = 0; j < func_call->n_args; j++) {
+                ASR::call_arg_t arg = func_call->m_args[j];
+                ASR::expr_t* arg_expr = arg.m_value;
+                handle_Var(arg_expr, &(func_call->m_args[j].m_value));
             }
         }
 
         void visit_Function(const ASR::Function_t& x) {
             ASR::Function_t* func = (ASR::Function_t*)(&x);
+            scope = func->m_symtab;
+            ASRUtils::SymbolDuplicator symbol_duplicator(al);
+            std::map<std::string, ASR::symbol_t*> scope_ = scope->get_scope();
+            std::vector<std::string> symbols_to_duplicate;
+            for (auto it: scope_) {
+                if (changed_external_function_symbol.find(it.first) != changed_external_function_symbol.end() &&
+                    is_external_sym_changed(it.second, changed_external_function_symbol[it.first])) {
+                    symbols_to_duplicate.push_back(it.first);
+                }
+            }
+
+            for (auto it: symbols_to_duplicate) {
+                scope->erase_symbol(it);
+                symbol_duplicator.duplicate_symbol(changed_external_function_symbol[it], scope);
+            }
+
             for (size_t i = 0; i < func->n_args; i++) {
                 ASR::expr_t* arg_expr = func->m_args[i];
                 if (ASR::is_a<ASR::Var_t>(*arg_expr)) {
@@ -194,7 +251,10 @@ void update_call_args(Allocator &al, SymbolTable *current_scope, bool implicit_i
                     ASR::symbol_t* arg_sym_underlying = ASRUtils::symbol_get_past_external(arg_sym);
                     ASR::symbol_t* sym = fetch_sym(arg_sym_underlying);
                     if (sym != arg_sym) {
-                        func->m_args[i] = ASRUtils::EXPR(ASR::make_Var_t(al, arg_expr->base.loc, sym));
+                        ASR::expr_t** current_expr_copy = current_expr;
+                        current_expr = const_cast<ASR::expr_t**>(&(func->m_args[i]));
+                        this->call_replacer_(sym);
+                        current_expr = current_expr_copy;
                     }
                 }
             }
@@ -211,7 +271,7 @@ void update_call_args(Allocator &al, SymbolTable *current_scope, bool implicit_i
     };
 
     if (implicit_interface) {
-        UpdateArgsVisitor v(al);
+        ArgsVisitor v(al, changed_external_function_symbol);
         SymbolTable *tu_symtab = ASRUtils::get_tu_symtab(current_scope);
         ASR::asr_t* asr_ = tu_symtab->asr_owner;
         ASR::TranslationUnit_t* tu = ASR::down_cast2<ASR::TranslationUnit_t>(asr_);
@@ -423,7 +483,7 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
         ASR::ttype_t* member_type = ASRUtils::TYPE(ASR::make_Struct_t(al,
             member_variable->base.base.loc, mem_es));
         return ASR::make_StructInstanceMember_t(al, loc, ASRUtils::EXPR(v_var),
-                mem_es, member_type, nullptr);
+            mem_es, ASRUtils::fix_scoped_type(al, member_type, current_scope), nullptr);
     } else {
         LCOMPILERS_ASSERT(ASR::is_a<ASR::Variable_t>(*member));
         ASR::Variable_t* member_variable = ASR::down_cast<ASR::Variable_t>(member);
@@ -513,7 +573,7 @@ ASR::asr_t* getStructInstanceMember_t(Allocator& al, const Location& loc,
             }
         }
         return ASR::make_StructInstanceMember_t(al, loc, ASRUtils::EXPR(v_var),
-            member_ext, member_type, value);
+            member_ext, ASRUtils::fix_scoped_type(al, member_type, current_scope), value);
     }
 }
 
@@ -881,7 +941,7 @@ void process_overloaded_assignment_function(ASR::symbol_t* proc, ASR::expr_t* ta
             ASRUtils::insert_module_dependency(a_name, al, current_module_dependencies);
             ASRUtils::set_absent_optional_arguments_to_null(a_args, subrout, al);
             asr = ASRUtils::make_SubroutineCall_t_util(al, loc, a_name, sym,
-                                            a_args.p, 2, nullptr, nullptr, false);
+                                            a_args.p, 2, nullptr, nullptr, false, false);
         }
     }
 }
@@ -914,7 +974,7 @@ bool use_overloaded_assignment(ASR::expr_t* target, ASR::expr_t* value,
         ASR::symbol_t* orig_sym = ASRUtils::symbol_get_past_external(sym);
         ASR::CustomOperator_t* gen_proc = ASR::down_cast<ASR::CustomOperator_t>(orig_sym);
         for( size_t i = 0; i < gen_proc->n_procs && !found; i++ ) {
-            ASR::symbol_t* proc = gen_proc->m_procs[i];
+            ASR::symbol_t* proc = ASRUtils::symbol_get_past_external(gen_proc->m_procs[i]);
             switch( proc->type ) {
                 case ASR::symbolType::Function: {
                     process_overloaded_assignment_function(proc, target, value, target_type,
@@ -933,7 +993,7 @@ bool use_overloaded_assignment(ASR::expr_t* target, ASR::expr_t* value,
                     break;
                 }
                 default: {
-                    err("Only functions and class procedures can be used for generic assignment statement", loc);
+                    err("Only functions and class procedures can be used for generic assignment statement, found " + std::to_string(proc->type), loc);
                 }
             }
         }
@@ -975,7 +1035,7 @@ bool use_overloaded(ASR::expr_t* left, ASR::expr_t* right,
                     ASR::down_cast<ASR::ClassProcedure_t>(
                     gen_proc->m_procs[i])->m_proc);
             } else {
-                proc = gen_proc->m_procs[i];
+                proc = ASRUtils::symbol_get_past_external(gen_proc->m_procs[i]);
             }
             switch(proc->type) {
                 case ASR::symbolType::Function: {
@@ -1141,30 +1201,6 @@ bool select_func_subrout(const ASR::symbol_t* proc, const Vec<ASR::call_arg_t>&
     return result;
 }
 
-int select_generic_procedure(const Vec<ASR::call_arg_t>& args,
-        const ASR::GenericProcedure_t &p, Location loc,
-        const std::function<void (const std::string &, const Location &)> err,
-        bool raise_error) {
-    for (size_t i=0; i < p.n_procs; i++) {
-        if( ASR::is_a<ASR::ClassProcedure_t>(*p.m_procs[i]) ) {
-            ASR::ClassProcedure_t *clss_fn
-                = ASR::down_cast<ASR::ClassProcedure_t>(p.m_procs[i]);
-            const ASR::symbol_t *proc = ASRUtils::symbol_get_past_external(clss_fn->m_proc);
-            if( select_func_subrout(proc, args, loc, err) ) {
-                return i;
-            }
-        } else {
-            if( select_func_subrout(p.m_procs[i], args, loc, err) ) {
-                return i;
-            }
-        }
-    }
-    if( raise_error ) {
-        err("Arguments do not match for any generic procedure, " + std::string(p.m_name), loc);
-    }
-    return -1;
-}
-
 ASR::asr_t* symbol_resolve_external_generic_procedure_without_eval(
             const Location &loc,
             ASR::symbol_t *v, Vec<ASR::call_arg_t>& args,
@@ -1223,7 +1259,7 @@ ASR::asr_t* symbol_resolve_external_generic_procedure_without_eval(
         }
         return ASRUtils::make_SubroutineCall_t_util(al, loc, final_sym,
                                         v, args.p, args.size(),
-                                        nullptr, nullptr, false);
+                                        nullptr, nullptr, false, false);
     } else {
         if( func ) {
             ASRUtils::set_absent_optional_arguments_to_null(args, func, al);
@@ -1314,7 +1350,9 @@ ASR::asr_t* make_Cast_t_value(Allocator &al, const Location &a_loc,
 
 ASR::symbol_t* import_class_procedure(Allocator &al, const Location& loc,
         ASR::symbol_t* original_sym, SymbolTable *current_scope) {
-    if( original_sym && ASR::is_a<ASR::ClassProcedure_t>(*original_sym) ) {
+    if( original_sym && (ASR::is_a<ASR::ClassProcedure_t>(*original_sym) ||
+        (ASR::is_a<ASR::Variable_t>(*original_sym) &&
+         ASR::is_a<ASR::FunctionType_t>(*ASRUtils::symbol_type(original_sym)))) ) {
         std::string class_proc_name = ASRUtils::symbol_name(original_sym);
         if( original_sym != current_scope->resolve_symbol(class_proc_name) ) {
             std::string imported_proc_name = "1_" + class_proc_name;
@@ -1403,6 +1441,7 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
     Vec<ASR::expr_t*> shape_args;
     shape_args.reserve(al, 1);
     shape_args.push_back(al, expr1);
+    bool is_value_character_array = ASR::is_a<ASR::Character_t>(*ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(ASRUtils::expr_type(expr2))));
 
     Vec<ASR::dimension_t> dims;
     dims.reserve(al, 1);
@@ -1415,10 +1454,11 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
     dims.push_back(al, dim);
     ASR::ttype_t* dest_shape_type = ASRUtils::TYPE(ASR::make_Array_t(al, loc,
         ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)), dims.p, dims.size(),
-        ASR::array_physical_typeType::FixedSizeArray));
+        is_value_character_array ? ASR::array_physical_typeType::CharacterArraySinglePointer: ASR::array_physical_typeType::FixedSizeArray));
 
     ASR::expr_t* dest_shape = nullptr;
     ASR::expr_t* value = nullptr;
+    ASR::ttype_t* ret_type = nullptr;
     if( ASRUtils::is_fixed_size_array(expr1_mdims, expr1_ndims) ) {
         Vec<ASR::expr_t*> lengths; lengths.reserve(al, expr1_ndims);
         for( size_t i = 0; i < expr1_ndims; i++ ) {
@@ -1440,8 +1480,8 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
         if( ASRUtils::is_value_constant(expr2) &&
             ASRUtils::get_fixed_size_of_array(expr1_mdims, expr1_ndims) <= 256 ) {
             ASR::ttype_t* value_type = ASRUtils::TYPE(ASR::make_Array_t(al, loc,
-                ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)), dims.p, dims.size(),
-                ASR::array_physical_typeType::FixedSizeArray));
+                ASRUtils::type_get_past_array(ASRUtils::expr_type(expr2)), dims.p, dims.size(),
+                is_value_character_array ? ASR::array_physical_typeType::CharacterArraySinglePointer: ASR::array_physical_typeType::FixedSizeArray));
             Vec<ASR::expr_t*> values;
             values.reserve(al, ASRUtils::get_fixed_size_of_array(expr1_mdims, expr1_ndims));
             for( int64_t i = 0; i < ASRUtils::get_fixed_size_of_array(expr1_mdims, expr1_ndims); i++ ) {
@@ -1449,6 +1489,7 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
             }
             value = ASRUtils::EXPR(ASR::make_ArrayConstant_t(al, loc,
                 values.p, values.size(), value_type, ASR::arraystorageType::ColMajor));
+            ret_type = value_type;
         }
     } else {
         dest_shape = ASRUtils::EXPR(ASR::make_IntrinsicArrayFunction_t(al, loc,
@@ -1456,7 +1497,23 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
             shape_args.size(), 0, dest_shape_type, nullptr));
     }
 
-    expr2 = ASRUtils::EXPR(ASR::make_ArrayBroadcast_t(al, loc, expr2, dest_shape, expr1_type, value));
+    if (ret_type == nullptr) {
+        // TODO: Construct appropriate return type here
+        // For now simply coping the type from expr1
+        if (ASRUtils::is_simd_array(expr1)) {
+            // TODO: Make this more general; do not check for SIMDArray
+            ret_type = ASRUtils::duplicate_type(al, expr1_type);
+        } else {
+            ret_type = expr1_type;
+        }
+    }
+    expr2 = ASRUtils::EXPR(ASR::make_ArrayBroadcast_t(al, loc, expr2, dest_shape, ret_type, value));
+
+    if (ASRUtils::extract_physical_type(expr1_type) != ASRUtils::extract_physical_type(ret_type)) {
+        expr2 = ASRUtils::EXPR(ASRUtils::make_ArrayPhysicalCast_t_util(al, loc, expr2,
+            ASRUtils::extract_physical_type(ret_type),
+            ASRUtils::extract_physical_type(expr1_type), expr1_type, nullptr));
+    }
 }
 
 void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
@@ -1484,10 +1541,12 @@ void make_ArrayBroadcast_t_util(Allocator& al, const Location& loc,
     }
 }
 
-int64_t compute_trailing_zeros(int64_t number) {
+int64_t compute_trailing_zeros(int64_t number, int64_t kind) {
     int64_t trailing_zeros = 0;
-    if (number == 0) {
+    if (number == 0 && kind == 4) {
         return 32;
+    } else if (number == 0 && kind == 8) {
+        return 64;
     }
     while (number % 2 == 0) {
         number = number / 2;
@@ -1496,6 +1555,24 @@ int64_t compute_trailing_zeros(int64_t number) {
     return trailing_zeros;
 }
 
+int64_t compute_leading_zeros(int64_t number, int64_t kind) {
+    int64_t leading_zeros = 0;
+    int64_t total_bits = 32;
+    if (kind == 8) total_bits = 64;
+    if (number < 0) return 0;
+    while (total_bits > 0) {
+        if (number%2 == 0) {
+            leading_zeros++;
+        } else {
+            leading_zeros = 0; 
+        }
+        number = number/2;
+        total_bits--;
+    }
+    return leading_zeros;
+}
+
+
 //Initialize pointer to zero so that it can be initialized in first call to get_instance
 ASRUtils::LabelGenerator* ASRUtils::LabelGenerator::label_generator = nullptr;
 
