@@ -19,16 +19,12 @@ interface dimag
     module procedure zaimag
 end interface
 
-interface sqrt
-    module procedure ssqrt, dsqrt, csqrt, zsqrt
-end interface
-
 interface exp
     module procedure sexp, dexp, cexp, zexp
 end interface
 
 interface log
-    module procedure slog, dlog, clog, zlog, alog
+    module procedure slog, dlog, clog, zlog
 end interface
 
 interface erf
@@ -39,10 +35,6 @@ interface erfc
     module procedure serfc, derfc
 end interface
 
-interface gamma
-    module procedure sgamma, dgamma
-end interface
-
 interface log_gamma
     module procedure slog_gamma, dlog_gamma
 end interface
@@ -229,40 +221,6 @@ end interface
 r = c_zaimag(x)
 end function
 
-! sqrt -------------------------------------------------------------------------
-
-elemental real(sp) function ssqrt(x) result(r)
-real(sp), intent(in) :: x
-r = _lfortran_sqrt(x)
-end function
-
-elemental real(dp) function dsqrt(x) result(r)
-real(dp), intent(in) :: x
-r = _lfortran_sqrt(x)
-end function
-
-elemental complex(sp) function csqrt(x) result(r)
-complex(sp), intent(in) :: x
-interface
-    pure complex(c_float) function c_csqrt(x) bind(c, name="_lfortran_csqrt")
-    import :: c_float
-    complex(c_float), intent(in), value :: x
-    end function
-end interface
-r = c_csqrt(x)
-end function
-
-elemental complex(dp) function zsqrt(x) result(r)
-complex(dp), intent(in) :: x
-interface
-    pure complex(c_double) function c_zsqrt(x) bind(c, name="_lfortran_zsqrt")
-    import :: c_double
-    complex(c_double), intent(in), value :: x
-    end function
-end interface
-r = c_zsqrt(x)
-end function
-
 ! exp --------------------------------------------------------------------------
 
 elemental real(sp) function sexp(x) result(r)
@@ -414,30 +372,6 @@ end interface
 r = c_derfc(x)
 end function
 
-! gamma ------------------------------------------------------------------------
-
-elemental real(sp) function sgamma(x) result(r)
-real(sp), intent(in) :: x
-interface
-    pure real(c_float) function c_sgamma(x) bind(c, name="_lfortran_sgamma")
-    import :: c_float
-    real(c_float), intent(in), value :: x
-    end function
-end interface
-r = c_sgamma(x)
-end function
-
-elemental real(dp) function dgamma(x) result(r)
-real(dp), intent(in) :: x
-interface
-    pure real(c_double) function c_dgamma(x) bind(c, name="_lfortran_dgamma")
-    import :: c_double
-    real(c_double), intent(in), value :: x
-    end function
-end interface
-r = c_dgamma(x)
-end function
-
 ! log_gamma --------------------------------------------------------------------
 
 elemental real(sp) function slog_gamma(x) result(r)
@@ -1144,7 +1078,7 @@ end subroutine
 ! system_clock------------------------------------------------------------------
 
 pure subroutine i32sys_clock(count, count_rate, count_max)
-integer(4), intent(out) :: count, count_rate, count_max
+integer(4), intent(out), optional :: count, count_rate, count_max
 interface
     pure subroutine c_i32sys_clock(count, count_rate, count_max) &
         bind(c, name="_lfortran_i32sys_clock")
@@ -1155,7 +1089,8 @@ call c_i32sys_clock(count, count_rate, count_max)
 end subroutine
 
 pure subroutine i64sys_clock(count, count_rate, count_max)
-integer(8), intent(out) :: count, count_rate, count_max
+integer(8), intent(out) :: count
+integer(8), intent(out), optional :: count_rate, count_max
 interface
     pure subroutine c_i64sys_clock(count, count_rate, count_max) &
         bind(c, name="_lfortran_i64sys_clock")
